# URLs
SEFFAFLIK_URL = "https://api.epias.com.tr/epias/exchange/transparency/"
# Requests Errors
__request_error = "İstek esnasında hata oluştu. İstek tekrar deneniyor!"
__requestsConnectionErrorLogging = "Bağlantı Hatası! Lütfen internet bağlantınızı kontrol ediniz!"
__requestsAuthenticationErrorLogging = "Kimlik dosyası oluşturulmamış yada geçersiz kimlik! Lütfen kimlik " \
                                       "bilgilerinizi kontrol edip kimlik dosyasını tekrar oluşturunuz!"
__requestsTimeoutErrorLogging = "İstek zaman aşımına uğradı! Web-Servis şartnamesinde belirtmiş olduğunuz IP " \
                                "adresini kullandığınızdan emin olunuz!"
# Requests timeout
__timeout = 1000000
